/*******************************************************************************
*
*	TITLE:		tracker.c 
*
*	VERSION:	1.0                           
*
*	DATE:		09-Jan-2004
*
*	AUTHOR:		R. Kevin Watson
*
*	COMMENTS:	This code assumes IR sensor 1 is the left sensor on the left tracker
*				         "        IR sensor 2 is the right sensor on the left tracker
*				         "        IR sensor 3 is the left sensor on the right tracker
*				         "        IR sensor 4 is the right sensor on the right tracker
*
*				         "        left IR beacon tracking servo motor is on PWM 1
*				         "        right IR beacon tracking servo motor is on PWM 2
*
*				         "        beacon type selector switch is on digital I/O 7
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	03-Dec-2003  0.1  RKW Original
*	07-Dec-2003  0.2  RKW - Modified Track_Beacon() to handle an arbitrary number
*	                  of IR beam tracking assemblies. All state machine
*	                  information is now kept in the Tracker_Data data structure.
*	                  RKW - Cleaned-up Track_Beacon to make it understandable, 
*	                  hiding most of the tough-to-grasp code in tracker.h
*	09-Jan-2004  1.0  RKW - This version used for kick-off demo
*
*******************************************************************************/



#include "ifi_picdefs.h"
#include "ifi_default.h"
#include "ifi_aliases.h"
#include "ifi_utilities.h"
//#include "printf_lib.h"
#include "print_routines.h"
#include "user_routines.h"
#include "receiver.h"
#include "tracker.h"


Tracker_Data_Struct Tracker_Data[2]; // two trackers: left and right

/*******************************************************************************
*
*	FUNCTION:		Initialize_Tracker()
*
*	PURPOSE:		Initializes the infrared beacon tracker code.
*
*	CALLED FROM:	User_Initialization()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		Adjustable tracker parameters are located in tracker.h

*******************************************************************************/
void Initialize_Tracker(void)
{
	// initialize servo PWM channels
	LEFT_TRACKER_SERVO = INITIAL_SERVO_POSITION; // rotate left tracker fully clockwise
	RIGHT_TRACKER_SERVO = INITIAL_SERVO_POSITION; // rotate right tracker fully clockwise

	// initialize the left tracker's state machine
	Tracker_Data[LEFT].Status = NONE_IN_VIEW; // defined in tracker.h
	Tracker_Data[LEFT].Position = INITIAL_SERVO_POSITION; // defined in tracker.h
	Tracker_Data[LEFT].Search_Increment = INITIAL_SERVO_STEP_SIZE; // defined in tracker.h
	Tracker_Data[LEFT].Beacon_Type = BEACON_TYPE; // defined in tracker.h
	Tracker_Data[LEFT].Left_Sensor_Stats_Index = LEFT_TRACKER_LEFT_SENSOR; // defined in tracker.h
	Tracker_Data[LEFT].Right_Sensor_Stats_Index = LEFT_TRACKER_RIGHT_SENSOR; // defined in tracker.h

	// initialize the right tracker's state machine
	Tracker_Data[RIGHT].Status = NONE_IN_VIEW; // defined in tracker.h
	Tracker_Data[RIGHT].Position = INITIAL_SERVO_POSITION; // defined in tracker.h
	Tracker_Data[RIGHT].Search_Increment = INITIAL_SERVO_STEP_SIZE; // defined in tracker.h
	Tracker_Data[RIGHT].Beacon_Type = BEACON_TYPE; // defined in tracker.h
	Tracker_Data[RIGHT].Left_Sensor_Stats_Index = RIGHT_TRACKER_LEFT_SENSOR; // defined in tracker.h
	Tracker_Data[RIGHT].Right_Sensor_Stats_Index = RIGHT_TRACKER_RIGHT_SENSOR; // defined in tracker.h
}

/*******************************************************************************
*
*	FUNCTION:		Track_Beacon()
*
*	PURPOSE:		Controls the servo motor(s) such that the pair of
*					infrared beacon sensors mounted on it will always
*					point at the selected IR beacon. Using two or more
*					beacon trackers, separated by a known baseline
*					distance, and some trigonometry will allow the user
*					to calculate the heading and distance to the beacon.
*
*	CALLED FROM:	navigate.c/navigate()
*
*	PARAMETERS:		unsigned char that identifies which tracker to use
*
*	RETURNS:		nothing		
*
*	COMMENTS:		There must be one Tracker_Data structure per tracker.
*					More than two trackers can be used.
*
*******************************************************************************/

void Track_Beacon(unsigned char Tracker)
{
	// which state was tracker <Tracker> in the last time through?
	switch(TRACKER_STATUS)
	{
		case NONE_IN_VIEW: // neither sensor sees the beacon
		{
			if(LEFT_SENSOR > 0)
			{
				// found the beacon
				TRACKER_STATUS = LEFT_IN_VIEW;
				PrintString((char *) "Tracker: from none to left ");
                PrintCR();
			}
			else
			{
				// continue to rotate left until the beacon comes into view
				TRACKER_POSITION += SEARCH_INCREMENT; // rolling over is okay
			}
			break;
		}
		case LEFT_IN_VIEW: // left sensor sees beacon, but the right sensor doesn't
		{			
			if(RIGHT_SENSOR > 0)
			{
				// found the beacon
				TRACKER_STATUS = BOTH_IN_VIEW;
				PrintString((char *) "Tracker: from left to both ");
                PrintCR();
			}
			else
			{
				// continue to rotate left until the beacon comes into view of the right sensor
				TRACKER_POSITION += SEARCH_INCREMENT; // rolling over is okay
			}
			break;
		}	
		case RIGHT_IN_VIEW:	// right sensor sees beacon, but the left sensor doesn't
		{
			if(LEFT_SENSOR > 0)
			{
				// found the beacon
				TRACKER_STATUS = BOTH_IN_VIEW;
				PrintString((char *) "Tracker: from right to both ");
                PrintCR();
			}
			else
			{
				// continue to rotate right until the beacon comes into view of the left sensor
				TRACKER_POSITION -= SEARCH_INCREMENT; // rolling over is okay
			}
			break;
		}
		case BOTH_IN_VIEW: // both sensors see the beacon
		{
			if((LEFT_SENSOR > 0) &&	(RIGHT_SENSOR > 0))
			{
				// do nothing because both sensors see the beacon
				PrintString((char *) "Tracker: both ");
                PrintCR();
			}
			else
			{
				// is the beacon in view of the left sensor?
				if(LEFT_SENSOR > 0)
				{
					// left sensor sees beacon; change state and rotate left
					TRACKER_STATUS = LEFT_IN_VIEW;
					TRACKER_POSITION += SEARCH_INCREMENT; // rolling over is okay
					PrintString((char *) "Tracker: from both to left ");
                    PrintCR();
				}
				// is the beacon in view of the right sensor?
				else if(RIGHT_SENSOR > 0)
				{
					// right sensor sees beacon; change state and rotate right
					TRACKER_STATUS = RIGHT_IN_VIEW;
					TRACKER_POSITION -= SEARCH_INCREMENT; // rolling over is okay
					PrintString((char *) "Tracker: from both to right ");
                    PrintCR();
				}
				else
				{
					// neither sensor sees the beacon; change state and start search
					TRACKER_STATUS = NONE_IN_VIEW;
					TRACKER_POSITION += SEARCH_INCREMENT; // rolling over is okay
				    PrintString((char *) "Tracker: from both to none ");
                    PrintCR();
				}
			}

			break;
		} 

	}// end switch(TRACKER_STATUS)

}


